local _G = _G
local Cartographer3 = _G.Cartographer3
local Cartographer3_Data = Cartographer3.Data

local groupPOIs = {}

local function groupMemberPOI_AddDataToFullTooltip(self)
	Cartographer3.Utils.AddUnitDataToFullTooltip(self.unit)
end
local function groupMemberPOI_AddDataToTooltipLine(self)
	Cartographer3.Utils.AddUnitDataToTooltipLine(self.unit)
end

local nextGroupMemberColorUpdate = 0
local flashing = true
function Cartographer3.ShowGroupMembers(elapsed, currentTime)
	if not Cartographer3_Data.currentZoneData then
		return
	end
	local prefix, amount = "raid", GetNumRaidMembers()
	local playerRaidID = 0
	if amount == 0 then
		prefix, amount = "party", GetNumPartyMembers()
	else
		playerRaidID = UnitInRaid("player") + 1
	end
	local zoneLeft = Cartographer3_Data.currentZoneData.mapLeft
	local zoneWidth = Cartographer3_Data.currentZoneData.mapWidth
	local zoneBottom = Cartographer3_Data.currentZoneData.mapBottom
	local zoneHeight = Cartographer3_Data.currentZoneData.mapHeight
	for i = 1, amount do
		local poi = groupPOIs[i]
		if not poi then
			poi = CreateFrame("Frame", Cartographer3_Data.mapView:GetName() .. "_GroupMember" .. i, Cartographer3_Data.mapView)
			groupPOIs[i] = poi
			Cartographer3.AddPOI(poi)
			poi.AddDataToFullTooltip = groupMemberPOI_AddDataToFullTooltip
			poi.AddDataToTooltipLine = groupMemberPOI_AddDataToTooltipLine
			poi:SetWidth(1)
			poi:SetHeight(1)
			local outer = poi:CreateTexture(nil--[[poi:GetName() .. "_OuterTexture"]], "BORDER")
			poi.outer = outer
			outer:SetAllPoints()
			outer:SetTexture([[Interface\AddOns\Cartographer3\Artwork\PartyOuter]])
			local inner = poi:CreateTexture(nil--[[poi:GetName() .. "_InnerTexture"]], "ARTWORK")
			poi.inner = inner
			inner:SetAllPoints()
			inner:SetTexture([[Interface\AddOns\Cartographer3\Artwork\PartyInner]])
			local text = poi:CreateTexture(nil--[[poi:GetName() .. "_Text"]], "OVERLAY")
			poi.text = text
			text:SetAllPoints()
			text:SetVertexColor(0, 0, 0)
		end
		if i ~= playerRaidID then
			local unit = prefix .. i
			poi.unit = unit
			local x, y = GetPlayerMapPosition(unit)
			if x == 0 and y == 0 then
				poi:Hide()
			else
				poi:Show()
			
				poi:SetPoint("CENTER", x * zoneWidth + zoneLeft, (1 - y) * zoneHeight + zoneBottom)
			end
		else
			poi:Hide()
		end
	end
	for i = amount+1, #groupPOIs do
		local poi = groupPOIs[i]
		poi:Hide()
	end
	if currentTime > nextGroupMemberColorUpdate then
		nextGroupMemberColorUpdate = currentTime + 0.5
		flashing = not flashing
		for i = 1, amount do
			local poi = groupPOIs[i]
			if poi:IsShown() then
				local poi_inner = poi.inner
				local poi_outer = poi.outer
				local unit = poi.unit
				if prefix == "raid" then
					local _, _, subgroup = GetRaidRosterInfo(i)
					poi.text:SetTexture([[Interface\AddOns\Cartographer3\Artwork\Group]] .. subgroup)
					poi.text:Show()
				else
					poi.text:Hide()
				end
				if flashing then
					if UnitAffectingCombat(unit) then
						poi_outer:SetVertexColor(unpack(Cartographer3_Data.COMBAT_COLOR))
					elseif UnitIsDeadOrGhost(unit) then
						poi_outer:SetVertexColor(unpack(Cartographer3_Data.DEAD_COLOR))
					elseif MapUnit_IsInactive(unit) then
						poi_outer:SetVertexColor(unpack(Cartographer3_Data.INACTIVE_COLOR))
					else
						poi_outer:SetVertexColor(unpack(Cartographer3_Data.NORMAL_STATUS_COLOR))
					end
				else
					poi_outer:SetVertexColor(unpack(Cartographer3_Data.NORMAL_STATUS_COLOR))
				end
				if not done then
					local _, class = UnitClass(unit)
					local classColor = RAID_CLASS_COLORS[class]
					poi_inner:SetVertexColor(classColor.r, classColor.g, classColor.b)
				end
			end
		end
	end
end

Cartographer3.Utils.AddTimer(Cartographer3.ShowGroupMembers)
